'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

class VuexModule {
    constructor(module) {
        this.actions = module.actions;
        this.mutations = module.mutations;
        this.state = module.state;
        this.getters = module.getters;
        this.namespaced = module.namespaced;
        this.modules = module.modules;
    }
}
function getModule(moduleClass) {
    const statics = moduleClass.constructor._statics;
    if (!statics) {
        throw new Error(`ERR_GET_MODULE_NO_STATICS : Could not get module accessor. 
      Make sure your module is dynamic and has name,
      i.e. @Module({dynamic: true, name: 'something' })`);
    }
    return statics;
}

function stateFactory(module) {
    const state = new (module.prototype.constructor)({});
    const s = {};
    Object.keys(state).forEach((key) => {
        if (['actions', 'getters', 'mutations', 'modules', 'state', 'namespaced'].indexOf(key) !== -1) {
            // These are reserved keys, cannot use these
            return;
        }
        if (state.hasOwnProperty(key)) {
            if (typeof state[key] !== 'function') {
                s[key] = state[key];
            }
        }
    });
    return s;
}

/*! *****************************************************************************
Copyright (c) Microsoft Corporation. All rights reserved.
Licensed under the Apache License, Version 2.0 (the "License"); you may not use
this file except in compliance with the License. You may obtain a copy of the
License at http://www.apache.org/licenses/LICENSE-2.0

THIS CODE IS PROVIDED ON AN *AS IS* BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
KIND, EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION ANY IMPLIED
WARRANTIES OR CONDITIONS OF TITLE, FITNESS FOR A PARTICULAR PURPOSE,
MERCHANTABLITY OR NON-INFRINGEMENT.

See the Apache Version 2.0 License for specific language governing permissions
and limitations under the License.
***************************************************************************** */

function __awaiter(thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
}

function staticStateGenerator(module, modOpt, statics) {
    Object.keys(module.state).forEach((key) => {
        if (module.state.hasOwnProperty(key)) {
            // If not undefined or function means it is a state value
            if (['undefined', 'function'].indexOf(typeof module.state[key]) === -1) {
                Object.defineProperty(statics, key, {
                    get() {
                        return modOpt.store.state[modOpt.name][key];
                    }
                });
            }
        }
    });
}
function staticGetterGenerator(module, modOpt, statics) {
    Object.keys(module.getters).forEach((key) => {
        if (module.namespaced) {
            Object.defineProperty(statics, key, {
                get() {
                    return modOpt.store.getters[`${modOpt.name}/${key}`];
                }
            });
        }
        else {
            Object.defineProperty(statics, key, {
                get() {
                    return modOpt.store.getters[key];
                }
            });
        }
    });
}
function staticMutationGenerator(module, modOpt, statics) {
    Object.keys(module.mutations).forEach((key) => {
        if (module.namespaced) {
            statics[key] = function (...args) {
                modOpt.store.commit(`${modOpt.name}/${key}`, ...args);
            };
        }
        else {
            statics[key] = function (...args) {
                modOpt.store.commit(key, ...args);
            };
        }
    });
}
function staticActionGenerators(module, modOpt, statics) {
    Object.keys(module.actions).forEach((key) => {
        if (module.namespaced) {
            statics[key] = function (...args) {
                return __awaiter(this, void 0, void 0, function* () {
                    return yield modOpt.store.dispatch(`${modOpt.name}/${key}`, ...args);
                });
            };
        }
        else {
            statics[key] = function (...args) {
                return __awaiter(this, void 0, void 0, function* () {
                    return yield modOpt.store.dispatch(key, ...args);
                });
            };
        }
    });
}

function moduleDecoratorFactory(moduleOptions) {
    return function (constructor) {
        const module = constructor;
        const stateFactory$$1 = () => stateFactory(module);
        if (!module.state) {
            module.state = (moduleOptions && moduleOptions.stateFactory) ? stateFactory$$1 : stateFactory$$1();
        }
        if (!module.getters) {
            module.getters = {};
        }
        module.namespaced = moduleOptions && moduleOptions.namespaced;
        Object.getOwnPropertyNames(module.prototype).forEach((funcName) => {
            const descriptor = Object.getOwnPropertyDescriptor(module.prototype, funcName);
            if (descriptor.get) {
                module.getters[funcName] = (moduleState) => descriptor.get.call(moduleState);
            }
        });
        if (moduleOptions.dynamic) {
            const modOpt = moduleOptions;
            if (!modOpt.name) {
                throw new Error('Name of module not provided in decorator options');
            }
            let statics = {};
            // ===========  For statics ==============
            // ------ state -------
            staticStateGenerator(module, modOpt, statics);
            // ------- getters -------
            if (module.getters) {
                staticGetterGenerator(module, modOpt, statics);
            }
            // -------- mutations --------
            if (module.mutations) {
                staticMutationGenerator(module, modOpt, statics);
            }
            // -------- actions ---------
            if (module.actions) {
                staticActionGenerators(module, modOpt, statics);
            }
            modOpt.store.registerModule(modOpt.name, // TODO: Handle nested modules too in future
            module);
            Object.defineProperty(constructor, '_statics', {
                value: statics
            });
            return constructor;
        }
    };
}
function Module(modOrOpt) {
    if (typeof modOrOpt === 'function') {
        /*
         * @Module decorator called without options (directly on the class definition)
         */
        moduleDecoratorFactory({})(modOrOpt);
    }
    else {
        /*
         * @Module({...}) decorator called with options
         */
        return moduleDecoratorFactory(modOrOpt);
    }
}

function actionDecoratorFactory(params) {
    return function (target, key, descriptor) {
        const module = target.constructor;
        if (!module.actions) {
            module.actions = {};
        }
        const actionFunction = descriptor.value;
        const action = function (context, payload) {
            return __awaiter(this, void 0, void 0, function* () {
                try {
                    const actionPayload = yield actionFunction.call(context, payload);
                    if (params) {
                        if (params.commit) {
                            context.commit(params.commit, actionPayload);
                        }
                    }
                }
                catch (e) {
                    console.error('Could not perform action ' + key.toString());
                    console.error(e);
                }
            });
        };
        module.actions[key] = action;
    };
}
/**
 * /**
 * The @Action decorator turns an async function into an Vuex action
 *
 * @param targetOrParams the module class
 * @param key name of the action
 * @param descriptor the action function descriptor
 * @constructor
 */
function Action(targetOrParams, key, descriptor) {
    if (!key && !descriptor) {
        /*
         * This is the case when `targetOrParams` is params.
         * i.e. when used as -
         * <pre>
            @Action({commit: 'incrCount'})
            async getCountDelta() {
              return 5
            }
         * </pre>
         */
        return actionDecoratorFactory(targetOrParams);
    }
    else {
        /*
         * This is the case when @Action is called on action function
         * without any params
         * <pre>
         *   @Action
         *   async doSomething() {
         *    ...
         *   }
         * </pre>
         */
        actionDecoratorFactory()(targetOrParams, key, descriptor);
    }
}

function Mutation(target, key, descriptor) {
    const module = target.constructor;
    if (!module.mutations) {
        module.mutations = {};
    }
    const mutationFunction = descriptor.value;
    const mutation = function (state, payload) {
        mutationFunction.call(state, payload);
    };
    module.mutations[key] = mutation;
}

function MutationAction(params) {
    return function (target, key, descriptor) {
        const module = target.constructor;
        if (!module.mutations) {
            module.mutations = {};
        }
        if (!module.actions) {
            module.actions = {};
        }
        const mutactFunction = descriptor.value;
        const action = function (context, payload) {
            return __awaiter(this, void 0, void 0, function* () {
                try {
                    const actionPayload = yield mutactFunction.call(context, payload);
                    context.commit(key, actionPayload);
                }
                catch (e) {
                    console.error('Could not perform action ' + key.toString());
                    console.error(e);
                }
            });
        };
        const mutation = function (state, payload) {
            for (let stateItem of params.mutate) {
                if (state[stateItem] != null && payload[stateItem] != null) {
                    state[stateItem] = payload[stateItem];
                }
            }
        };
        module.actions[key] = action;
        module.mutations[key] = mutation;
    };
}

exports.VuexModule = VuexModule;
exports.Module = Module;
exports.Action = Action;
exports.Mutation = Mutation;
exports.MutationAction = MutationAction;
exports.getModule = getModule;
//# sourceMappingURL=index.js.map
